@@ -30,24 +30,30 @@ API
 
 import os
 import dis
-import types
+import sys
 import inspect
 import textwrap
+import warnings
 import functools
 import contextlib
-from types import MappingProxyType, FunctionType
+from types import MappingProxyType, FunctionType, MethodType, CodeType
 from itertools import zip_longest
 from typing import (
-    Callable, Any, Optional, Union, Sized, Mapping, Tuple, Type, TypeVar, TYPE_CHECKING
+    Callable, Any, Optional, Union, Sized, Mapping, Tuple, Type, TypeVar, cast, TYPE_CHECKING
 )
 
-from .signature import generate_signature, _signature_to_str, _get_cls_annotation
+from .signature_utils import generate_signature, _signature_to_str, _get_cls_annotation
 
 if TYPE_CHECKING:
     from numpy import ndarray
 else:
     ndarray = 'numpy.ndarray'
 
+if sys.version_info <= (3, 6):
+    COMMA = ','
+else:
+    COMMA = ''
+
 __all__ = ['get_sphinx_domain', 'create_assertion_doc', 'bind_callable', 'skip_if',
            'len_eq', 'str_eq', 'function_eq']
 
@@ -84,7 +90,7 @@ def bind_callable(class_type: Union[type, Any], func: Callable,
     :rtype: :data:`None`
 
     """
-    name = name if name is not None else func.__name__
+    name_: str = name if name is not None else func.__name__
 
     # Create the new function
     function, signature_str = _create_assertion_func(func)
@@ -100,10 +106,10 @@ def bind_callable(class_type: Union[type, Any], func: Callable,
 
     # Set the new method
     if isinstance(class_type, type):  # A class
-        setattr(class_type, name, function)
+        setattr(class_type, name_, function)
     else:  # A class instance
-        method = types.MethodType(function, class_type)  # Create a bound method
-        setattr(class_type, name, method)
+        method = MethodType(function, class_type)  # Create a bound method
+        setattr(class_type, name_, method)
 
 
 def _set_annotations(func_new: Callable, func_old: Callable) -> None:
@@ -115,7 +121,7 @@ def _set_annotations(func_new: Callable, func_old: Callable) -> None:
     annotations['post_process'] = Optional[Callable[[Any], Any]]
     annotations['message'] = Optional[str]
 
-    # Create an additional annotation incase **func_old** is an instance- or class-method
+    # Create an additional annotation in case **func_old** is an instance-method
     with contextlib.suppress(ValueError):  # Raised if **func_old** has no readable signature
         prm = inspect.signature(func_old).parameters
         if 'self' in prm:
@@ -123,7 +129,7 @@ def _set_annotations(func_new: Callable, func_old: Callable) -> None:
             annotations[key] = value
 
 
-def _create_assertion_func(func: Callable) -> Tuple[types.FunctionType, str]:
+def _create_assertion_func(func: Callable) -> Tuple[FunctionType, str]:
     """Generate the assertion function for :func:`bind_callable`.
 
     Parameters
@@ -132,7 +138,7 @@ def _create_assertion_func(func: Callable) -> Tuple[types.FunctionType, str]:
         A callable object forming the basis of the to-be created assertion function.
 
     """
-    _empty = inspect._empty
+    _empty = inspect._empty  # type: ignore
 
     def _to_str(prm: inspect.Parameter) -> str:
         """Create a string from **prm**; ensure that callables are represented by their __name__."""
@@ -147,23 +153,26 @@ def _create_assertion_func(func: Callable) -> Tuple[types.FunctionType, str]:
     sgn_str2: str = _signature_to_str(sgn, 'func')
 
     # Create the code object for the to-be returned function
-    code_compile = compile(
+    code_compile: CodeType = compile(
         f'def {func.__name__}{sgn_str1}: __tracebackhide__ = True; self.assert_{sgn_str2}',
         "<string>", "exec"
     )
-    for code in code_compile.co_consts:
-        if isinstance(code, types.CodeType):
+    for code in code_compile.co_consts:  # Type: CodeType
+        if isinstance(code, CodeType):
             break
+    else:
+        raise TypeError(f"Failed to identify a {CodeType.__name__!r} instance "
+                        "in the compiled code")
 
     # Extract the default arguments for positional or keyword parameters
     defaults: Optional[Tuple[object, ...]] = code_compile.co_consts[-1]
     if isinstance(defaults, str):  # no default arguments
         defaults = None
-    func_new = types.FunctionType(code, {'func': func}, func.__name__, defaults)
+    func_new = FunctionType(code, {'func': func}, func.__name__, defaults)
 
     # Set default values for keyword-only parameters
-    KO = inspect.Parameter.KEYWORD_ONLY
-    kwdefault = {k: v.default for k, v in sgn.parameters.items() if v.kind is KO}
+    ko = inspect.Parameter.KEYWORD_ONLY
+    kwdefault = {k: v.default for k, v in sgn.parameters.items() if v.kind is ko}
     if kwdefault:
         func_new.__kwdefaults__ = kwdefault
 
@@ -200,7 +209,6 @@ See also
 --------
 {domain}:
 {summary}
-
 """
 
 
@@ -214,35 +222,44 @@ def create_assertion_doc(func: Callable, signature: Optional[str] = None) -> str
     --------
     .. code:: python
 
-        >>> docstring: str = wrap_docstring(isinstance)
+        >>> from assertionlib.functions import create_assertion_doc
+
+        >>> docstring: str = create_assertion_doc(isinstance)
         >>> print(docstring)
         Perform the following assertion: :code:`assert isinstance(*args, **kwargs)`.
-
+        <BLANKLINE>
         Parameters
         ----------
         invert : :class:`bool`
             Invert the output of the assertion: :code:`assert not isinstance(*args, **kwargs)`.
             This value should only be supplied as keyword argument.
-
+        <BLANKLINE>
         exception : :class:`type` [:exc:`Exception`], optional
             Assert that **exception** is raised during/before the assertion operation.
             This value should only be supplied as keyword argument.
-
+        <BLANKLINE>
         post_process : :class:`~collections.abc.Callable`, optional
             Apply post-processing to the to-be asserted data before asserting aforementioned data.
             Example functions would be the likes of :func:`~builtins.any` and :func:`~builtins.all`.
-
+        <BLANKLINE>
+        message : :data:`~typing.Any`, optional
+            A custom error message to-be passed to the ``assert`` statement.
+        <BLANKLINE>
         \*args/\**kwargs : :data:`~typing.Any`
             Parameters for catching excess variable positional and keyword arguments.
-
+        <BLANKLINE>
+        <BLANKLINE>
+        :rtype: :data:`None`
+        <BLANKLINE>
         See also
         --------
         :func:`~builtins.isinstance`:
             Return whether an object is an instance of a class or of a subclass thereof.
-
+        <BLANKLINE>
             A tuple, as in ``isinstance(x, (A, B, ...))``, may be given as the target to
             check against. This is equivalent to ``isinstance(x, A) or isinstance(x, B)
             or ...`` etc.
+        <BLANKLINE>
 
     Parameters
     ----------
@@ -258,23 +275,18 @@ def create_assertion_doc(func: Callable, signature: Optional[str] = None) -> str
     :class:`str`
         A new docstring constructed from **funcs'** docstring.
 
-    """  # noqa
+    """
     # domain is :class:`...`, :func:`...` or :meth:`...`
     domain = get_sphinx_domain(func)
-    sgn = signature if signature is not None else '(*args, **kwargs)'
+    sgn: str = signature if signature is not None else '(*args, **kwargs)'
 
     # Create a summary for a single `See Also` section using the docstring of **func**
     indent = 4 * ' '
-    func_doc = getattr(func, '__doc__', 'No description.')
-    if func_doc is None:
-        func_doc = 'No description.'
+    func_doc: str = func.__doc__ or 'No description.'
     func_summary = textwrap.indent(func_doc, indent)
 
     # Return a new docstring
-    try:
-        name = func.__qualname__
-    except AttributeError:
-        name = func.__name__
+    name: str = getattr(func, '__qualname__', func.__name__)
     return BASE_DOCSTRING.format(name=name, signature=sgn, domain=domain, summary=func_summary)
 
 
@@ -289,7 +301,7 @@ MODULE_DICT: Mapping[str, str] = MappingProxyType({
 def _is_builtin_func(func: Callable) -> bool:
     """Check if **func** is a builtin function."""
     try:
-        return inspect.isbuiltin(func) and '.' not in func.__qualname__
+        return inspect.isbuiltin(func) and '.' not in getattr(func, '__qualname__', '')
     except AttributeError:
         return False
 
@@ -302,6 +314,7 @@ def get_sphinx_domain(func: Callable, module_mapping: Mapping[str, str] = MODULE
     .. code:: python
 
         >>> from collections import OrderedDict
+        >>> from assertionlib.functions import get_sphinx_domain
 
         >>> value1: str = get_sphinx_domain(int)
         >>> print(value1)
@@ -309,7 +322,7 @@ def get_sphinx_domain(func: Callable, module_mapping: Mapping[str, str] = MODULE
 
         >>> value2: str = get_sphinx_domain(list.count)
         >>> print(value2)
-        :meth:`~list.count`
+        :meth:`~builtins.list.count`
 
         >>> value3: str = get_sphinx_domain(OrderedDict)
         >>> print(value3)
@@ -340,13 +353,13 @@ def get_sphinx_domain(func: Callable, module_mapping: Mapping[str, str] = MODULE
         Raised if **func** is neither a class or a (builtin) function or method.
 
     """
-    name = getattr(func, '__qualname__', func.__name__)
+    name: str = getattr(func, '__qualname__', func.__name__)
 
     # Extract the __module__ from **func**
     try:
         _module = func.__module__
     except AttributeError:  # Unbound methods don't have the `__module__` attribute
-        _module = func.__objclass__.__module__
+        _module = func.__objclass__.__module__  # type: ignore
 
     # Convert the extracted __module__ into an actual valid module
     module = MODULE_DICT.get(_module, _module)
@@ -358,14 +371,15 @@ def get_sphinx_domain(func: Callable, module_mapping: Mapping[str, str] = MODULE
         return f':meth:`~{module}.{name}`'
     elif inspect.isclass(func):
         return f':class:`~{module}.{name}`'
-    raise TypeError(f"{repr(name)} is neither a (builtin) function, method nor class")
+    raise TypeError(f"{name!r} is neither a (builtin) function, method nor class")
 
 
 #: An immutable mapping of to-be replaced substrings and their replacements.
 README_MAPPING: Mapping[str, str] = MappingProxyType({'``': '|', '()': ''})
 
 
-def load_readme(readme: str = 'README.rst', replace: Mapping[str, str] = README_MAPPING,
+def load_readme(readme: Union[str, os.PathLike] = 'README.rst',
+                replace: Mapping[str, str] = README_MAPPING,
                 **kwargs: Any) -> str:
     r"""Load and return the content of a readme file located in the same directory as this file.
 
@@ -390,42 +404,64 @@ def load_readme(readme: str = 'README.rst', replace: Mapping[str, str] = README_
     """
     readme_abs: str = os.path.join(os.path.dirname(__file__), readme)
     with open(readme_abs, 'r') as f:
-        ret = f.read(**kwargs)
+        ret: str = f.read(**kwargs)
     for old, new in replace.items():
         ret = ret.replace(old, new)
     return ret
 
 
 UserFunc = Callable[..., T]
+NoneFunc = Callable[..., None]
 
 
-def skip_if(condition: Any) -> Callable[[UserFunc], UserFunc]:
-    """A decorator which causes function calls to be ignored if :code:`bool(condition) is True`.
+def skip_if(condition: Any) -> Callable[[UserFunc], Union[UserFunc, NoneFunc]]:
+    f"""A decorator which causes function calls to be ignored if :code:`bool(condition) is True`.
+
+    A :exc:`UserWarning` is issued if **condition** evaluates to ``True``.
 
     Examples
     --------
     .. code:: python
 
-        >>> condition: bool = True
+        >>> import warnings
+        >>> from assertionlib.functions import skip_if
+
+        >>> condition = Exception("Error")
 
         >>> def func1() -> None:
         ...     print(True)
 
         >>> @skip_if(condition)
-        >>> def func2() -> None:
+        ... def func2() -> None:
         ...     print(True)
 
         >>> func1()
         True
-        >>> func2()
+
+        >>> with warnings.catch_warnings():  # Convert the warning into a raised exception
+        ...     warnings.simplefilter("error", UserWarning)
+        ...     func2()
+        Traceback (most recent call last):
+          ...
+        UserWarning: Exception('Error'{COMMA}) evaluated to True; skipping call to func2(...)
 
     """
-    def skip() -> None: pass
 
-    def decorator(func: UserFunc) -> UserFunc:
+    def skip() -> None:
+        return None
+
+    def decorator(func: UserFunc) -> Union[UserFunc, NoneFunc]:
         @functools.wraps(func)
-        def wrapper():
-            return func() if not condition else skip()
+        def wrapper(*args: Any, **kwargs: Any) -> Optional[T]:
+            if not condition:
+                return cast(T, func(*args, **kwargs))
+
+            exc = UserWarning(f"{condition!r:.70} evaluated to True; skipping call to {func.__name__}(...)")  # noqa: E501
+            if isinstance(condition, BaseException):
+                exc.__cause__ = condition
+
+            warnings.warn(exc, stacklevel=2)
+            return cast(None, skip())
         return wrapper
     return decorator
 
@@ -478,10 +514,9 @@ def function_eq(func1: FunctionType, func2: FunctionType) -> bool:
         code1 = func1.__code__
         code2 = func2.__code__
     except AttributeError as ex:
-        tb = ex.__traceback__
         name, obj = ('func1', func1) if code1 is None else ('func2', func2)
-        raise TypeError(f"'{name}' expected a function or object with the '__code__' attribute; "
-                        f"observed type: '{obj.__class__.__name__}'").with_traceback(tb)
+        raise TypeError(f"{name!r} expected a function or object with the '__code__' attribute; "
+                        f"observed type: {obj.__class__.__name__!r}") from ex
 
     iterator = zip_longest(dis.get_instructions(code1), dis.get_instructions(code2))
     tup_list = [(_sanitize_instruction(i), _sanitize_instruction(j)) for i, j in iterator]
