@@ -54,18 +54,33 @@ API
 
 """  # noqa
 
-import types
+import sys
 import inspect
 import reprlib
 import builtins
 import textwrap
-from typing import Any, Dict, Callable, Union, Tuple, Optional, TYPE_CHECKING
+from typing import Any, Dict, Callable, Union, Tuple, Optional, Mapping, List, TYPE_CHECKING
 from itertools import chain, islice
 
 if TYPE_CHECKING:
-    from scm.plams import Molecule, Atom, Bond, Settings
+    from scm.plams import Molecule, Atom, Bond, Settings  # type: ignore
     from numpy import ndarray
-    from pandas import DataFrame, Series
+    from pandas import DataFrame, Series  # type: ignore
+
+    from types import (BuiltinFunctionType, BuiltinMethodType, ModuleType, FunctionType, MethodType)
+    if sys.version_info >= (3, 7):
+        from types import MethodDescriptorType
+
+    else:
+        from typing_extensions import Protocol
+
+        class MethodDescriptorType(Protocol):
+            """See https://github.com/python/typeshed/blob/master/stdlib/3/types.pyi ."""
+            __name__: str
+            __qualname__: str
+            __objclass__: type
+            def __call__(self, *args: Any, **kwargs: Any) -> Any: ...  # noqa: E302
+            def __get__(self, obj: Any, type: type = ...) -> Any: ...  # noqa: E302
 else:
     Molecule = 'scm.plams.mol.molecule.Molecule'
     Atom = 'scm.plams.mol.molecule.Atom'
@@ -75,6 +90,13 @@ else:
     Series = 'pandas.core.series.Series'
     DataFrame = 'pandas.core.frame.DataFrame'
 
+    BuiltinFunctionType = "builtins.builtin_function_or_method"
+    BuiltinMethodType = "builtins.builtin_function_or_method"
+    ModuleType = "builtins.module"
+    FunctionType = "builtins.function"
+    MethodType = "builtins.method"
+    MethodDescriptorType = "builtins.method_descriptor"
+
 try:
     import numpy as np
     NUMPY_EX: Optional[ImportError] = None
@@ -89,7 +111,7 @@ except ImportError as ex:
 
 __all__ = ['NDRepr', 'aNDRepr']
 
-BuiltinType = Union[types.BuiltinFunctionType, types.BuiltinMethodType]
+BuiltinType = Union[BuiltinFunctionType, BuiltinMethodType]
 
 
 class NDRepr(reprlib.Repr):
@@ -183,7 +205,7 @@ class NDRepr(reprlib.Repr):
 
     """
 
-    def __init__(self, **kwargs: Any) -> None:
+    def __init__(self, **kwargs: Union[int, Mapping[str, Any]]) -> None:
         """Initialize a :class:`NDRepr` instance."""
         super().__init__()
         self.maxstring: int = 80
@@ -197,8 +219,8 @@ class NDRepr(reprlib.Repr):
         self.maxDataFrame: int = 12
         self.maxMolecule: int = 6
         self.maxSettings: int = self.maxdict
-        self.np_printoptions: Dict[str, Any] = {}
-        self.pd_printoptions: Dict[str, Any] = {}
+        self.np_printoptions: Mapping[str, Any] = {}
+        self.pd_printoptions: Mapping[str, Any] = {}
 
         # Update attributes based on **kwargs; raise an error if a key is unrecognized
         for k, v in kwargs.items():
@@ -211,6 +233,7 @@ class NDRepr(reprlib.Repr):
         if isinstance(obj, Exception):  # Refer all exceptions NDRepr.repr_Exception()
             return self.repr_Exception(obj, level)
         return super().repr1(obj, level)
+
     repr1.__doc__ = reprlib.Repr.repr1.__doc__
 
     def repr_float(self, obj: float, level: int) -> str:
@@ -220,7 +243,7 @@ class NDRepr(reprlib.Repr):
             return f'{obj:{i}.{i}e}'
         return f'{obj:{i}.{i}f}'  # Exponential notation
 
-    def repr_Exception(self, obj: Exception, level: int) -> str:
+    def repr_Exception(self, obj: Exception, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of an :exc`Exception` instance."""
         value = str(obj)
         i = self.maxException
@@ -230,17 +253,17 @@ class NDRepr(reprlib.Repr):
 
     # New methods for parsing callables
 
-    def repr_method(self, obj: types.MethodType, level: int) -> str:
+    def repr_method(self, obj: MethodType, level: int) -> str:
         """Create a :class:`str` representation of a bound method."""
         name, signature = self._parse_callable(obj, level)
         return f"<bound method '{name}{signature}'>"
 
-    def repr_method_descriptor(self, obj: 'types.MethodDescriptorType', level: int) -> str:
+    def repr_method_descriptor(self, obj: MethodDescriptorType, level: int) -> str:
         """Create a :class:`str` representation of an unbound method."""
         name, signature = self._parse_callable(obj, level)
         return f"<method '{name}{signature}'>"
 
-    def repr_function(self, obj: types.FunctionType, level: int) -> str:
+    def repr_function(self, obj: FunctionType, level: int) -> str:
         """Create a :class:`str` representation of a function."""
         name, signature = self._parse_callable(obj, level)
         return f"<function '{name}{signature}'>"
@@ -257,11 +280,11 @@ class NDRepr(reprlib.Repr):
         name, signature = self._parse_callable(obj, level)
         return f"<class '{name}{signature}'>"
 
-    def repr_module(self, obj: types.ModuleType, level: int) -> str:
+    def repr_module(self, obj: ModuleType, level: int) -> str:
         """Create a :class:`str` representation of a module."""
         return f"<module '{obj.__name__}'>"
 
-    def repr_Signature(self, obj: inspect.Signature, level: int) -> str:
+    def repr_Signature(self, obj: inspect.Signature, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a :class:`~inspect.Signature` instance."""
         i = self.maxSignature
         signature = str(obj)
@@ -290,15 +313,12 @@ class NDRepr(reprlib.Repr):
     def _parse_callable(self, obj: Callable, level: int) -> Tuple[str, str]:
         """Create a :class:`str` representation of the name and signature of a callable."""
         # Construct the name of the callable
-        try:
-            name = obj.__qualname__
-        except AttributeError:
-            name = obj.__name__
+        name: str = getattr(obj, '__qualname__', obj.__name__)
 
         # Construct the signature
         try:
             _signature = inspect.signature(obj)
-            signature = self.repr1(_signature, level - 1)
+            signature: str = self.repr1(_signature, level - 1)
         except ValueError:
             signature = '(...)'
 
@@ -306,7 +326,7 @@ class NDRepr(reprlib.Repr):
 
     # New PLAMS-related methods
 
-    def repr_Molecule(self, obj: Molecule, level: int) -> str:
+    def repr_Molecule(self, obj: Molecule, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a |plams.Molecule| instance."""
         if level <= 0:
             return f'{obj.__class__.__name__}(...)'
@@ -342,7 +362,7 @@ class NDRepr(reprlib.Repr):
         indent = 4 * ' '
         return f'{obj.__class__.__name__}(\n{textwrap.indent(ret[:-1], indent)}\n)'
 
-    def repr_Settings(self, obj: Settings, level: int) -> str:
+    def repr_Settings(self, obj: Settings, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a |plams.Settings| instance."""
         n = len(obj)
         if not obj:
@@ -350,7 +370,7 @@ class NDRepr(reprlib.Repr):
         elif level <= 0:
             return '\n...'
 
-        pieces = []
+        pieces: List[str] = []
         indent = 4 * ' '
         newlevel = level - 1
 
@@ -371,14 +391,14 @@ class NDRepr(reprlib.Repr):
             return f'{obj.__class__.__name__}(\n{textwrap.indent(ret[1:], indent)}\n)'
         return ret
 
-    def repr_Atom(self, obj: Atom, level: int) -> str:
+    def repr_Atom(self, obj: Atom, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a |plams.Atom| instance."""
         decimal = self.maxfloat
         space = 14 - (6 - decimal)  # The default PLAMS values for space and decimal are 14 and 6
         ret = obj.str(decimal=decimal, space=space).strip()
         return f'{obj.__class__.__name__}({ret})'
 
-    def repr_Bond(self, obj: Bond, level: int) -> str:
+    def repr_Bond(self, obj: Bond, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a |plams.Bond| instance."""
         return f'{obj.__class__.__name__}({obj})'
 
@@ -400,7 +420,7 @@ class NDRepr(reprlib.Repr):
         with np.printoptions(**kwargs):
             return builtins.repr(obj)
 
-    def repr_DataFrame(self, obj: DataFrame, level: int) -> str:
+    def repr_DataFrame(self, obj: DataFrame, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a :class:`pandas.DataFrame` instance."""
         if PANDAS_EX is not None:
             raise PANDAS_EX
@@ -416,7 +436,7 @@ class NDRepr(reprlib.Repr):
         with pd.option_context(*args):
             return builtins.repr(obj)
 
-    def repr_Series(self, obj: Series, level: int) -> str:
+    def repr_Series(self, obj: Series, level: int) -> str:  # noqa: N802
         """Create a :class:`str` representation of a :class:`pandas.Series` instance."""
         if PANDAS_EX is not None:
             raise PANDAS_EX
@@ -431,7 +451,7 @@ class NDRepr(reprlib.Repr):
         with pd.option_context(*args):
             return builtins.repr(obj)
 
-    def _get_ndformatter(self, obj: ndarray) -> Dict[str, Callable[..., str]]:
+    def _get_ndformatter(self, obj: ndarray) -> Dict[str, Callable[[Union[int, float]], str]]:
         """Return a value for the **formatter** argument in :func:`numpy.printoptions`."""
         if obj.dtype != float and obj.dtype != int:
             return {}
